# 公开版代码
# 使用MNIST数据集进行手写字体数字的识别的demo，演示CNN方法进行分类的调用方法
# 本程序2024.04定型，后续更新都在完整版代码中
# 本代码地址及代码说明：http://khsci.com/docs/index.php/2024/05/28/cnnregpy/
# 本代码知乎讲解：https://zhuanlan.zhihu.com/p/699703404
# 注意！本代码需要使用Python3.11版本，需要安装conda，环境配置请参考：https://zhuanlan.zhihu.com/p/693620785，以及本文件夹中的代码说明.docx
# 
import numpy as np
from khCNNreg import FunRegCNNs

# 1.读取 housing.txt 文件
data = np.loadtxt('housing.txt')

# 2.提取输入变量和输出变量
X = data[:, :13]  # 通过数组切片操作,将数据的前13列提取出来,赋值给变量X
y = data[:, 13]   # 通过数组切片操作,将数据的第14列提取出来,赋值给变量y
X = X.reshape(X.shape[0], 1, 1, X.shape[1])  # 对输入数据进行转置
divideR = [0.7, 0.1, 0.2]  # 设置数据集划分比例 训练集:验证集:测试集

# 3.设置网络结构
cLayer = np.array([[3, 1, 32, 1, 2]]) # 卷积层结构
poolingLayer = [['none', 0, 0, 2, 0]] # 设置池化层结构
fcLayer = []  # 设置全连接层结构

# 4.设置网络训练选项
options = {
'MaxEpochs': 10,          # 最大训练轮数为10
'MiniBatchSize': 64,       # 小批量数据大小为64
'InitialLearnRate': 0.01,  # 初始学习率为0.01
'ValidationFrequency': 5,  # 每5个epoch进行一次验证
'LearnRateDropPeriod': 10, # 每10个epoch学习率下降一次
'LearnRateDropFactor': 0.9 # 学习率下降因子为0.9
}

# 5.设置其他选项
setting = {
    'figflag': 'on',    # 显示图形
    'deviceSel': 'gpu', # 使用GPU作为设备
    'seed': 42,         # 随机种子设置为42
    'minmax': True      # 进行最小最大值归一化
}

# 6.调用 FunRegCNNs 函数进行测试
foreData, foreDataTrain, model, info = FunRegCNNs(X, y, divideR, cLayer, poolingLayer, fcLayer, options, setting)
"""
    使用CNN进行回归预测的快速实现函数
    
    参数:
    - dataX: 输入数据,形状为(num_samples, num_channels, height, width)的numpy数组
    - dataY: 输出结果,考虑可以为多维的numpy数组
    - divideR: 数据集划分比例,形如[train_ratio, val_ratio, test_ratio]的列表
    - cLayer: 卷积层结构,形状为(num_conv_layers, 5)的numpy数组,每一行代表一个卷积层的参数[filter_height, filter_width, num_filters, stride, padding]
    - poolingLayer: 池化层结构,形状为(num_conv_layers, 5)的列表,每一行代表一个池化层的参数['pool_type', pool_height, pool_width, stride, padding],其中pool_type可以是'maxPooling2dLayer'或'averagePooling2dLayer'或'none'
    - fcLayer: 全连接层结构,形状为(num_fc_layers,)的列表,每一个元素代表一个全连接层的输出维度,如果为空列表则只有一个输出维度等于1的全连接层
    - options: 网络训练相关的选项,字典类型,包含以下键值对:
      - 'solverName': 优化器类型,可以是'sgdm'或'rmsprop'或'adam',默认为'adam'
      - 'MaxEpochs': 最大迭代次数,默认为30
      - 'MiniBatchSize': 批量大小,默认为128
      - 'InitialLearnRate': 初始学习率,默认为0.005
      - 'ValidationFrequency': 验证频率,即每多少次迭代进行一次验证,默认为50
      - 'LearnRateSchedule': 学习率调度方式,可以是'piecewise'或'none',默认为'none'
      - 'LearnRateDropPeriod': 学习率下降周期,默认为10
      - 'LearnRateDropFactor': 学习率下降因子,默认为0.95
    - setting: 其他选项,字典类型,包含以下键值对:
      - figflag: 是否绘制图像,'on'为绘制,'off'为不绘制
      - deviceSel: 训练设备选择,可以是'cpu'或'gpu',默认为'gpu',当设置为'gpu'时,如果gpu硬件不可用,则会自动切换到cpu
      - seed: 随机种子,整数,设置为0时不启用,设置为其他整数时启用,不同的整数为不同的种子值,变换种子值会影响结果,相同种子的计算结果是一致的,缺省时为不设置随机种子
      - minmax: 是否进行归一化,布尔值,默认为True
      
    返回值:
    - foreData: 测试集的回归预测结果
    - foreDataTrain: 训练集的回归预测结果
    - model: 训练好的PyTorch模型
    - info: 包含训练过程中的损失和准确率信息的字典
"""

# 关于完整版代码：
# 公开版代码的函数文件为pyd文件，可以被调用，但无法查看代码。
# 完整版代码中全部为py文件，py文件可以查看源码并自由修改，画图无水印
# 完整版最大迭代次数无限制，可绘制网络结构图
# 完整版可导出预测结果、训练好的网络模型、神经网络训练相关参数（如loss值）等


# 如果需要封装好的函数的源码，可在下述连接（完整版）获取：
# http://khsci.com/docs/index.php/2024/05/28/cnnregpy/
# 编程不易，感谢支持~
